#include <p32xxxx.h>
#include <plib.h>
#include "PIC32MX150pins.h"
#include "Nokia5110.h"

// DEVCFG3:
#pragma config IOL1WAY  = OFF       // Peripheral Pin Select Configuration, allow mult reconfig
#pragma config PMDL1WAY = OFF		// Peripheral Module Disable Config, allow mult reconfig
// DEVCFG2:
#pragma config FPLLODIV = DIV_2     // PLL Output Divider
#pragma config FPLLMUL  = MUL_20    // PLL Multiplier
#pragma config FPLLIDIV = DIV_2     // PLL Input Divider
// 8MHz / 2 * 20 / 2 = 40MHz
// DEVCFG1:
#pragma config FWDTEN   = OFF       // Watchdog Timer
#pragma config WDTPS    = PS1       // Watchdog Timer Postscale
#pragma config FCKSM    = CSDCMD    // Clock Switching & Fail Safe Clock Monitor
#pragma config FPBDIV   = DIV_1     // Peripheral Clock divisor
#pragma config OSCIOFNC = OFF       // CLKO Enable
#pragma config POSCMOD  = OFF       // Primary Oscillator
#pragma config IESO     = OFF       // Internal/External Switch-over
#pragma config FSOSCEN  = OFF       // Secondary Oscillator Enable (KLO was off)
#pragma config FNOSC    = FRCPLL    // Oscillator Selection
// DEVCFG0:
#pragma config CP       = OFF       // Code Protect
#pragma config BWP      = ON        // Boot Flash Write Protect
#pragma config PWP      = OFF       // Program Flash Write Protect
#pragma config ICESEL   = ICS_PGx1  // ICE/ICD Comm Channel Select
#pragma config JTAGEN   = OFF       // JTAG Enable
#pragma config DEBUG    = OFF       // Background Debugger Enable
#define SYS_FREQ 40000000L 			// Clock frequency is 40MHz, so CoreTimer will be 20MHz

// Function prototypes
void init_analog(void);
int read_analog(int order);
void init_rcservo(void);
void set_rcservo(int degrees);
int read_ultrasonic(void);

// Main function

int main(void) {
  CFGCONbits.JTAGEN = 0; // turn off JTAG, get back those pins for IO use

  // sedt max performance
  SYSTEMConfigPerformance(SYS_FREQ);
  INTEnableSystemMultiVectoredInt();

  // assign the pins to functions
  PIC32MX150_setup_pins();

  /////////////////////////////////
  // H bridge setup
  /////////////////////////////////
  // A1 is OC2, B3 is OC1
  // B2 is direction for OC2, B4 is direction for OC1
  // Timer2 at 20kHz for frequency
  LATBbits.LATB2 = 1; // forwards
  LATBbits.LATB4 = 0;
  TRISBbits.TRISB2 = 0;
  TRISBbits.TRISB4 = 0;

  // set up OC1 and OC2 to 20000Hz
  T2CON = 0; // turn off TMR2
  OC1CON = 0; //turn off OC1
  OC2CON = 0; //turn off OC2
  T2CONbits.TCKPS = 2; // set prescaler 1:4
  TMR2 = 0; // start TMR2 counting from zero.
  PR2 = 499; // set period match value
  /* so 20000Hz from 499 from (40MHz / 4 prescaler / 20000Hz) - 1 = 499 */
  OC1CONbits.OCM = 0b110; // enable PWM mode with no fault protection
  OC2CONbits.OCM = 0b110; // enable PWM mode with no fault protection
  OC1RS = 0;
  OC1R = 0;
  OC2RS = 0;
  OC2R = 0;
  T2CONbits.ON = 1; // turn on TMR2
  OC1CONbits.ON = 1; // turn OC1 back on
  OC2CONbits.ON = 1; // turn OC2 back on

  /////////////////////////////////
  // LCD setup
  /////////////////////////////////
  // A4 is SDO2
  // B15 is SCK2
  // B5 is DC
  // B10 is Reset
  // B7 is SCE
  LcdInitialize();
  LcdClear();

  /////////////////////////////////
  // IR distance sensor setup
  /////////////////////////////////
  // B9 is SDA1
  // B8 is SCL1

  /////////////////////////////////
  // RC servo setup
  /////////////////////////////////
  // B14 is OC3
  // Timer 3 is 50Hz for frequency
  init_rcservo();

  /////////////////////////////////
  // Analog input setup
  /////////////////////////////////
  // AN0 on A0
  // AN11 on B13
  // AN12 on B12
  ANSELAbits.ANSA0 = 1; // Use AN0 on A0 as analog input
  ANSELBbits.ANSB12 = 1; // Use AN12 on B12 as analog input
  ANSELBbits.ANSB13 = 1; // Use AN11 on B13 as analog input
  init_analog();

  /////////////////////////////////
  // 8 other pins setup
  /////////////////////////////////
  // TBD: 2 pins for ultrasonic
  // B6
  // B11
  // A2
  // A3

  // spin for 2 seconds
  //WriteCoreTimer(0);
  //LATBbits.LATB4 = 0; // right
  //OC1RS = 499; // max 499
  //while (ReadCoreTimer() < 40000000) {
  //}
  //OC1RS = 0;
  //WriteCoreTimer(0);
  //LATBbits.LATB4 = 1; // left
  //OC1RS = 499; // max 499
  //while (ReadCoreTimer() < 40000000) {
  //}
  //OC1RS = 0;

  // walk for 2 seconds
  //WriteCoreTimer(0);
  //OC2RS = 499; // max 499
  //while (ReadCoreTimer() < 40000000) {
  //}
  //OC2RS = 0;

  // walk at 3/4 speed
  OC2RS = 375; // max 499

  // read the line sensor
  char LCDmessage[11];
  int left, middle, right;
  while (1) {
    // start timer so that the loop takes 1/10 second
    WriteCoreTimer(0);

    left = read_analog(0); // A0
    middle = read_analog(1); // B13
    right = read_analog(2); // B12
    sprintf(LCDmessage, "%d", left);
    gotoXY(0, 0);
    LcdString(LCDmessage);
    sprintf(LCDmessage, "%d", middle);
    gotoXY(0, 1);
    LcdString(LCDmessage);
    sprintf(LCDmessage, "%d", right);
    gotoXY(0, 2);
    LcdString(LCDmessage);

    if(middle - 100 > left) {
      // turn left
      LATBbits.LATB4 = 1; // left
      OC1RS = 250; // max 499
      OC2RS = 0; // max 499
    }
    else if(middle - 100 > right) {
      // turn right
      LATBbits.LATB4 = 0; // right
      OC1RS = 250; // max 499
      OC2RS = 375; // max 499
    }
    else {
      // nothing
      OC1RS = 0;
      OC2RS = 375; // max 499
    }

    // wait the rest of the 1/10 second
    while (ReadCoreTimer() < 20000000/10) {
    }
  }
  return 0;
}

/////////////////////////////////
// Functions
/////////////////////////////////

void init_analog(void) {
  AD1CON1bits.FORM = 0b000; // Select 16-bit integer output
  AD1CON1bits.ASAM = 1; // Auto Sample
  AD1CON1bits.SSRC = 0b111; // Auto convert sampled data to digital
  AD1CON2bits.VCFG = 0b000; // Select the vrefs to be Vdd and Vss

  AD1CON2bits.CSCNA = 1; // Enable ADC Scaning

  AD1CSSL = 0b1100000000001; // Select AN0 AN11 AN12 for scanning
  AD1CON2bits.SMPI = 3 - 1; // Interrupt after the 3rd read if using interrupts
  /* In scanning mode, this number must be at least as big as
   * the number of channels you are scanning/
   */

  AD1CON3bits.ADRC = 0; // Use PBCLK as clock source
  // Tpb = 1/80MHz = 12.5ns
  // Sample at 100ksps to reduce noise.
  // For ADC minimum, use 2 Tad, so Tad = 10us / (12+2) = 714ns
  // Sample time is Tad*2, so Tsamp = 1.428us
  // ADC clock divisor = 714ns / (1/80MHz) = 57
  // Actual sample rate is 1/(57*(12+2)*12.5ns) = 100ksps
  // So ADCS is 57 and SAMC is 2
  AD1CON3bits.SAMC = 2;
  AD1CON3bits.ADCS = 57;
  AD1CON1bits.ADON = 1; // Turn the ADC on
}

// In this case AN0 AN11 and AN12 are active, so if order =
// 0 - get AN0 from A0
// 1 - get AN11 for B13
// 2 - get AN12 from B12

int read_analog(int order) {
  switch (order) {
    case 0:
      return ADC1BUF0;
      break;
    case 1:
      return ADC1BUF1;
      break;
    case 2:
      return ADC1BUF2;
      break;
  }
}

void init_rcservo(void) {
  // set a 50Hz timer for the PWM signal to the rc servo
  T3CON = 0; // turn off TMR3, T3CON<ON> = 0
  OC3CON = 0; // turn off OC3, OC3CON<ON> = 0
  T3CONbits.TCKPS = 0b111; // set prescaler 1:256
  TMR3 = 0; // start TMR3 counting from zero
  PR3 = 6249; // set period match value
  /* now TMR3 has a period of (PR3 + 1) / PBCLK * 2^TCON<TCKPS> = 20ms or 50Hz. */

  OC3CONbits.OCM = 0b110; // enable PWM mode with no fault protection
  /* 0 degrees on the servo is a 0.5ms pulse, or OC3RS = (6250/0.02s)*0.0005s = 157 */
  /* 180 degrees on the servo is a 2.5ms pulse, or OC3RS = (6250/0.02s)*0.0025s = 781 */
  /* 90 degrees on the servo is a 1.5ms pulse, or OC3RS = (6250/0.02s)*0.0015s = 467 */
  /* so DON'T SET OC3RS OUTSIDE THE RANGE 157-781*/
  OC3RS = 467; // duty cycle is OC3RS/(PR2+1)
  OC3R = 467; // set initial PWM duty cycle in counts
  // don't write to OC3R while PWM is running (it's read-only),
  // instead use OC3RS.

  T3CONbits.ON = 1; // turn on TMR3
  OC3CONbits.ON = 1; // turn on OC3
}

void set_rcservo(int degrees) {
  float set_valuef = degrees * 3.467 + 157.0; //(degrees-0)*(781-157)/(180-0)+157;
  int set_valuei = set_valuef;

  OC3RS = set_valuei;
}

int read_ultrasonic() {
  // read the ultrasonic rangefinder
  int flight_time, distance;

  // from last year

  return distance;
}

